/* elf32-m68hc05.c -- Motorola 68HC05 BFD support
   Copyright 2005 Free Software Foundation, Inc.
   Copyright 2005 Martin Kgler
   Contributed by Martin Kgler (mkoegler@auto.tuwien.ac.at)

   This file is part of BFD, the Binary File Descriptor library.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "bfd.h"
#include "sysdep.h"
#include "bfdlink.h"
#include "libbfd.h"
#include "elf-bfd.h"
#include "elf/m68hc05.h"
#include "aout/stab_gnu.h"

static reloc_howto_type elf_m68hc05_howto_table[] = {
  /* This reloc does nothing.  */
  HOWTO (R_M68HC05_NONE,	/* type */
	 0,			/* rightshift */
	 2,			/* size (0 = byte, 1 = short, 2 = long) */
	 32,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_dont,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_NONE",	/* name */
	 FALSE,			/* partial_inplace */
	 0,			/* src_mask */
	 0,			/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* A 8 bit absolute relocation */
  HOWTO (R_M68HC05_8,		/* type */
	 0,			/* rightshift */
	 0,			/* size (0 = byte, 1 = short, 2 = long) */
	 8,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_bitfield,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_8",		/* name */
	 FALSE,			/* partial_inplace */
	 0x00ff,		/* src_mask */
	 0x00ff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* A 16 bit absolute relocation */
  HOWTO (R_M68HC05_16,		/* type */
	 0,			/* rightshift */
	 1,			/* size (0 = byte, 1 = short, 2 = long) */
	 16,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_bitfield,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_16",	/* name */
	 FALSE,			/* partial_inplace */
	 0xffff,		/* src_mask */
	 0xffff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* A 32 bit absolute relocation. */
  HOWTO (R_M68HC05_32,		/* type */
	 0,			/* rightshift */
	 2,			/* size (0 = byte, 1 = short, 2 = long) */
	 32,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_bitfield,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_32",	/* name */
	 FALSE,			/* partial_inplace */
	 0xffffffff,		/* src_mask */
	 0xffffffff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */


  /* A 8 bit absolute relocation (upper address) */
  HOWTO (R_M68HC05_HI8,		/* type */
	 8,			/* rightshift */
	 0,			/* size (0 = byte, 1 = short, 2 = long) */
	 8,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_dont,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_HI8",	/* name */
	 FALSE,			/* partial_inplace */
	 0x00ff,		/* src_mask */
	 0x00ff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* A 8 bit absolute relocation (lower address) */
  HOWTO (R_M68HC05_LO8,		/* type */
	 0,			/* rightshift */
	 0,			/* size (0 = byte, 1 = short, 2 = long) */
	 8,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_dont,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_LO8",	/* name */
	 FALSE,			/* partial_inplace */
	 0x00ff,		/* src_mask */
	 0x00ff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* A 8 bit absolute relocation (upper address, high word) */
  HOWTO (R_M68HC05_HHI8,	/* type */
	 8,			/* rightshift */
	 0,			/* size (0 = byte, 1 = short, 2 = long) */
	 8,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_dont,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_HHI8",	/* name */
	 FALSE,			/* partial_inplace */
	 0x00ff,		/* src_mask */
	 0x00ff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* A 8 bit absolute relocation (lower address, high word) */
  HOWTO (R_M68HC05_HLO8,	/* type */
	 0,			/* rightshift */
	 0,			/* size (0 = byte, 1 = short, 2 = long) */
	 8,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_dont,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_HLO8",	/* name */
	 FALSE,			/* partial_inplace */
	 0x00ff,		/* src_mask */
	 0x00ff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* A 16-bit low relocation */
  HOWTO (R_M68HC05_HI16,	/* type */
	 0,			/* rightshift */
	 1,			/* size (0 = byte, 1 = short, 2 = long) */
	 16,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_dont,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_HI16",	/* name */
	 FALSE,			/* partial_inplace */
	 0xffff,		/* src_mask */
	 0xffff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */
  /* A 16-bit low relocation */
  HOWTO (R_M68HC05_LO16,	/* type */
	 0,			/* rightshift */
	 1,			/* size (0 = byte, 1 = short, 2 = long) */
	 16,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_dont,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_LO16",	/* name */
	 FALSE,			/* partial_inplace */
	 0xffff,		/* src_mask */
	 0xffff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */


  /* A 8 bit PC-rel relocation */
  HOWTO (R_M68HC05_PCREL_8,	/* type */
	 0,			/* rightshift */
	 0,			/* size (0 = byte, 1 = short, 2 = long) */
	 8,			/* bitsize */
	 TRUE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_bitfield,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_PCREL_8",	/* name */
	 FALSE,			/* partial_inplace */
	 0x00ff,		/* src_mask */
	 0x00ff,		/* dst_mask */
	 TRUE),			/* pcrel_offset */

  /* A 16 bit PC-rel relocation */
  HOWTO (R_M68HC05_PCREL_16,	/* type */
	 0,			/* rightshift */
	 1,			/* size (0 = byte, 1 = short, 2 = long) */
	 16,			/* bitsize */
	 TRUE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_bitfield,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_PCREL_16",	/* name */
	 FALSE,			/* partial_inplace */
	 0xffff,		/* src_mask */
	 0xffff,		/* dst_mask */
	 TRUE),			/* pcrel_offset */

  /* A 32 bit PC-rel relocation. */
  HOWTO (R_M68HC05_PCREL_32,	/* type */
	 0,			/* rightshift */
	 2,			/* size (0 = byte, 1 = short, 2 = long) */
	 32,			/* bitsize */
	 TRUE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_bitfield,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_PCREL_32",	/* name */
	 FALSE,			/* partial_inplace */
	 0xffffffff,		/* src_mask */
	 0xffffffff,		/* dst_mask */
	 TRUE),			/* pcrel_offset */

  /* A 8 bit relaxable absolute relocation */
  HOWTO (R_M68HC05_RELAX_8,	/* type */
	 0,			/* rightshift */
	 0,			/* size (0 = byte, 1 = short, 2 = long) */
	 8,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_bitfield,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_RELAX_8",	/* name */
	 FALSE,			/* partial_inplace */
	 0x00ff,		/* src_mask */
	 0x00ff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* A 16 bit relaxable absolute relocation */
  HOWTO (R_M68HC05_RELAX_16,	/* type */
	 0,			/* rightshift */
	 1,			/* size (0 = byte, 1 = short, 2 = long) */
	 16,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_bitfield,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_RELAX_16",	/* name */
	 FALSE,			/* partial_inplace */
	 0xffff,		/* src_mask */
	 0xffff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* This reloc marks the begin of an expanded jump and does nothing.  */
  HOWTO (R_M68HC05_RELAX_GROUP,	/* type */
	 0,			/* rightshift */
	 2,			/* size (0 = byte, 1 = short, 2 = long) */
	 32,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_dont,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_RELAX_GROUP",	/* name */
	 FALSE,			/* partial_inplace */
	 0,			/* src_mask */
	 0,			/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* A 8-bit relocation which stores to offset of a symbol relative to its section start */
  HOWTO (R_M68HC05_SECTION_OFFSET8,	/* type */
	 0,			/* rightshift */
	 0,			/* size (0 = byte, 1 = short, 2 = long) */
	 8,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_bitfield,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_SECTION_OFFSET8",	/* name */
	 FALSE,			/* partial_inplace */
	 0x00ff,		/* src_mask */
	 0x00ff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* A 16-bit relocation which stores to offset of a symbol relative to its section start */
  HOWTO (R_M68HC05_SECTION_OFFSET16,	/* type */
	 0,			/* rightshift */
	 1,			/* size (0 = byte, 1 = short, 2 = long) */
	 16,			/* bitsize */
	 FALSE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_bitfield,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_SECTION_OFFSET16",	/* name */
	 FALSE,			/* partial_inplace */
	 0xffff,		/* src_mask */
	 0xffff,		/* dst_mask */
	 FALSE),		/* pcrel_offset */

  /* A 8 bit PC-rel relocation */
  HOWTO (R_M68HC05_RELAX_PCREL_8,	/* type */
	 0,			/* rightshift */
	 0,			/* size (0 = byte, 1 = short, 2 = long) */
	 8,			/* bitsize */
	 TRUE,			/* pc_relative */
	 0,			/* bitpos */
	 complain_overflow_bitfield,	/* complain_on_overflow */
	 bfd_elf_generic_reloc,	/* special_function */
	 "R_M68HC05_RELAX_PCREL_8",	/* name */
	 FALSE,			/* partial_inplace */
	 0x00ff,		/* src_mask */
	 0x00ff,		/* dst_mask */
	 TRUE),			/* pcrel_offset */

};

struct m68hc05_reloc_map
{
  bfd_reloc_code_real_type bfd_reloc_val;
  unsigned char elf_reloc_val;
};

static const struct m68hc05_reloc_map m68hc05_reloc_map[] = {
  {BFD_RELOC_NONE, R_M68HC05_NONE},
  {BFD_RELOC_8, R_M68HC05_8},
  {BFD_RELOC_16, R_M68HC05_16},
  {BFD_RELOC_32, R_M68HC05_32},
  {BFD_RELOC_M68HC05_HI8, R_M68HC05_HI8},
  {BFD_RELOC_M68HC05_LO8, R_M68HC05_LO8},
  {BFD_RELOC_M68HC05_HHI8, R_M68HC05_HHI8},
  {BFD_RELOC_M68HC05_HLO8, R_M68HC05_HLO8},
  {BFD_RELOC_HI16, R_M68HC05_HI16},
  {BFD_RELOC_LO16, R_M68HC05_LO16},
  {BFD_RELOC_8_PCREL, R_M68HC05_PCREL_8},
  {BFD_RELOC_16_PCREL, R_M68HC05_PCREL_16},
  {BFD_RELOC_32_PCREL, R_M68HC05_PCREL_32},
  {BFD_RELOC_M68HC05_RELAX_8, R_M68HC05_RELAX_8},
  {BFD_RELOC_M68HC05_RELAX_16, R_M68HC05_RELAX_16},
  {BFD_RELOC_M68HC05_RELAX_GROUP, R_M68HC05_RELAX_GROUP},
  {BFD_RELOC_M68HC05_SECTION_OFFSET8, R_M68HC05_SECTION_OFFSET8},
  {BFD_RELOC_M68HC05_SECTION_OFFSET16, R_M68HC05_SECTION_OFFSET16},
  {BFD_RELOC_M68HC05_RELAX_PCREL_8, R_M68HC05_RELAX_PCREL_8},
};

static reloc_howto_type *
bfd_elf32_bfd_reloc_type_lookup (bfd * abfd ATTRIBUTE_UNUSED,
				 bfd_reloc_code_real_type code)
{
  unsigned int i;

  for (i = 0;
       i < sizeof (m68hc05_reloc_map) / sizeof (struct m68hc05_reloc_map);
       i++)
    {
      if (m68hc05_reloc_map[i].bfd_reloc_val == code)
	return &elf_m68hc05_howto_table[m68hc05_reloc_map[i].elf_reloc_val];
    }

  return NULL;
}

static reloc_howto_type *
bfd_elf32_bfd_reloc_name_lookup (bfd *abfd ATTRIBUTE_UNUSED,
				 const char *r_name)
{
  unsigned int i;

  for (i = 0;
       i < (sizeof (elf_m68hc05_howto_table)
	    / sizeof (elf_m68hc05_howto_table[0]));
       i++)
    if (elf_m68hc05_howto_table[i].name != NULL
	&& strcasecmp (elf_m68hc05_howto_table[i].name, r_name) == 0)
      return &elf_m68hc05_howto_table[i];

  return NULL;
}

/* Set the howto pointer for an M68HC05 ELF reloc.  */

static void
m68hc05_info_to_howto_rel (bfd * abfd ATTRIBUTE_UNUSED,
			   arelent * cache_ptr, Elf_Internal_Rela * dst)
{
  unsigned int r_type;

  r_type = ELF32_R_TYPE (dst->r_info);
  BFD_ASSERT (r_type < (unsigned int) R_M68HC05_max);
  cache_ptr->howto = &elf_m68hc05_howto_table[r_type];
}


static asection *
elf32_m68hc05_gc_mark_hook (asection * sec,
			    struct bfd_link_info *info ATTRIBUTE_UNUSED,
			    Elf_Internal_Rela * rel,
			    struct elf_link_hash_entry *h,
			    Elf_Internal_Sym * sym)
{
  if (h != NULL)
    {
      switch (ELF32_R_TYPE (rel->r_info))
	{
	default:
	  switch (h->root.type)
	    {
	    case bfd_link_hash_defined:
	    case bfd_link_hash_defweak:
	      return h->root.u.def.section;

	    case bfd_link_hash_common:
	      return h->root.u.c.p->section;

	    default:
	      break;
	    }
	}
    }
  else
    return bfd_section_from_elf_index (sec->owner, sym->st_shndx);

  return NULL;
}

static bfd_boolean
elf32_m68hc05_gc_sweep_hook (bfd * abfd ATTRIBUTE_UNUSED,
			     struct bfd_link_info *info ATTRIBUTE_UNUSED,
			     asection * sec ATTRIBUTE_UNUSED,
			     const Elf_Internal_Rela *
			     relocs ATTRIBUTE_UNUSED)
{
  /* We don't use got and plt entries for 68hc05.  */
  return TRUE;
}


/* Look through the relocs for a section during the first phase.*/

static bfd_boolean
elf32_m68hc05_check_relocs (bfd * abfd, struct bfd_link_info *info,
			    asection * sec, const Elf_Internal_Rela * relocs)
{
  Elf_Internal_Shdr *symtab_hdr;
  struct elf_link_hash_entry **sym_hashes;
  const Elf_Internal_Rela *rel;
  const Elf_Internal_Rela *rel_end;

  if (info->relocatable)
    return TRUE;

  symtab_hdr = &elf_tdata (abfd)->symtab_hdr;
  sym_hashes = elf_sym_hashes (abfd);

  rel_end = relocs + sec->reloc_count;

  for (rel = relocs; rel < rel_end; rel++)
    {
      struct elf_link_hash_entry *h;
      unsigned long r_symndx;

      r_symndx = ELF32_R_SYM (rel->r_info);

      if (r_symndx < symtab_hdr->sh_info)
	h = NULL;
      else
	h = sym_hashes[r_symndx - symtab_hdr->sh_info];

      switch (ELF32_R_TYPE (rel->r_info))
	{
	default:
	  ;
	}
    }

  return TRUE;
}

/* Perform a single relocation.  By default we use the standard BFD
   routines, but a few relocs, we have to do them ourselves.  */

static bfd_reloc_status_type
m68hc05_final_link_relocate (reloc_howto_type * howto, bfd * input_bfd,
			     asection * input_section,
			     bfd_byte * contents,
			     Elf_Internal_Rela * rel, bfd_vma relocation,
			     asection * sec)
{
  bfd_reloc_status_type r = bfd_reloc_ok;
  bfd_vma x;
  bfd_signed_vma srel;
  bfd_vma offset = 0;

  switch (howto->type)
    {
    case R_M68HC05_HI8:
      contents += rel->r_offset;
      srel = (bfd_signed_vma) relocation + rel->r_addend;
      x = (srel >> 8) & 0xff;
      bfd_put_8 (input_bfd, x, contents);
      break;
    case R_M68HC05_LO8:
      contents += rel->r_offset;
      srel = (bfd_signed_vma) relocation + rel->r_addend;
      x = (srel >> 0) & 0xff;
      bfd_put_8 (input_bfd, x, contents);
      break;
    case R_M68HC05_HHI8:
      contents += rel->r_offset;
      srel = (bfd_signed_vma) relocation + rel->r_addend;
      x = (srel >> 24) & 0xff;
      bfd_put_8 (input_bfd, x, contents);
      break;
    case R_M68HC05_HLO8:
      contents += rel->r_offset;
      srel = (bfd_signed_vma) relocation + rel->r_addend;
      x = (srel >> 16) & 0xff;
      bfd_put_8 (input_bfd, x, contents);
      break;
    case R_M68HC05_SECTION_OFFSET8:
    case R_M68HC05_SECTION_OFFSET16:
      if (!sec)
	sec = bfd_abs_section_ptr;
      relocation -= sec->output_section->vma;
      goto def;

    case R_M68HC05_PCREL_8:
    case R_M68HC05_PCREL_16:
    case R_M68HC05_PCREL_32:
    case R_M68HC05_RELAX_PCREL_8:
      offset = -(1 << howto->size);
    default:
    def:
      r = _bfd_final_link_relocate (howto, input_bfd, input_section,
				    contents, rel->r_offset,
				    relocation, rel->r_addend + offset);
    }

  return r;
}


/* Relocate an M68HC05 ELF section.  */
static bfd_boolean
elf32_m68hc05_relocate_section (bfd * output_bfd ATTRIBUTE_UNUSED,
				struct bfd_link_info *info,
				bfd * input_bfd, asection * input_section,
				bfd_byte * contents,
				Elf_Internal_Rela * relocs,
				Elf_Internal_Sym * local_syms,
				asection ** local_sections)
{
  Elf_Internal_Shdr *symtab_hdr;
  struct elf_link_hash_entry **sym_hashes;
  Elf_Internal_Rela *rel;
  Elf_Internal_Rela *relend;

  if (info->relocatable)
    return TRUE;

  symtab_hdr = &elf_tdata (input_bfd)->symtab_hdr;
  sym_hashes = elf_sym_hashes (input_bfd);
  relend = relocs + input_section->reloc_count;

  for (rel = relocs; rel < relend; rel++)
    {
      reloc_howto_type *howto;
      unsigned long r_symndx;
      Elf_Internal_Sym *sym;
      asection *sec;
      struct elf_link_hash_entry *h;
      bfd_vma relocation;
      bfd_reloc_status_type r;
      const char *name = NULL;
      int r_type;

      /* This is a final link.  */
      r_type = ELF32_R_TYPE (rel->r_info);
      r_symndx = ELF32_R_SYM (rel->r_info);
      howto = elf_m68hc05_howto_table + ELF32_R_TYPE (rel->r_info);
      h = NULL;
      sym = NULL;
      sec = NULL;

      if (r_symndx < symtab_hdr->sh_info)
	{
	  sym = local_syms + r_symndx;
	  sec = local_sections[r_symndx];
	  relocation = _bfd_elf_rela_local_sym (output_bfd, sym, &sec, rel);

	  name = bfd_elf_string_from_elf_section
	    (input_bfd, symtab_hdr->sh_link, sym->st_name);
	  name = (name == NULL) ? bfd_section_name (input_bfd, sec) : name;
	}
      else
	{
	  bfd_boolean unresolved_reloc, warned;

	  RELOC_FOR_GLOBAL_SYMBOL (info, input_bfd, input_section, rel,
				   r_symndx, symtab_hdr, sym_hashes,
				   h, sec, relocation,
				   unresolved_reloc, warned);
	}

      r = m68hc05_final_link_relocate (howto, input_bfd, input_section,
				       contents, rel, relocation, sec);

      if (r != bfd_reloc_ok)
	{
	  const char *msg = (const char *) NULL;

	  switch (r)
	    {
	    case bfd_reloc_overflow:
	      r = info->callbacks->reloc_overflow
		(info, (h ? &h->root : NULL),
		 name, howto->name, (bfd_vma) 0,
		 input_bfd, input_section, rel->r_offset);
	      break;

	    case bfd_reloc_undefined:
	      r = info->callbacks->undefined_symbol
		(info, name, input_bfd, input_section, rel->r_offset, TRUE);
	      break;

	    case bfd_reloc_outofrange:
	      msg = _("internal error: out of range error");
	      break;

	    case bfd_reloc_notsupported:
	      msg = _("internal error: unsupported relocation error");
	      break;

	    case bfd_reloc_dangerous:
	      msg = _("internal error: dangerous relocation");
	      break;

	    default:
	      msg = _("internal error: unknown error");
	      break;
	    }

	  if (msg)
	    r = info->callbacks->warning
	      (info, msg, name, input_bfd, input_section, rel->r_offset);

	  if (!r)
	    return FALSE;
	}
    }

  return TRUE;
}

/* This is a version of bfd_generic_get_relocated_section_contents
   which uses elf32_m68hc05_relocate_section.  */

static bfd_byte *
elf32_m68hc05_get_relocated_section_contents (bfd * output_bfd,
					      struct bfd_link_info *link_info,
					      struct bfd_link_order
					      *link_order, bfd_byte * data,
					      bfd_boolean relocatable,
					      asymbol ** symbols)
{
  Elf_Internal_Shdr *symtab_hdr;
  asection *input_section = link_order->u.indirect.section;
  bfd *input_bfd = input_section->owner;
  asection **sections = NULL;
  Elf_Internal_Rela *internal_relocs = NULL;
  Elf_Internal_Sym *isymbuf = NULL;

  /* We only need to handle the case of relaxing, or of having a
     particular set of section contents, specially.  */
  if (relocatable)
    return bfd_generic_get_relocated_section_contents (output_bfd, link_info,
						       link_order, data,
						       relocatable, symbols);

  if (elf_section_data (input_section)->this_hdr.contents == NULL)
    if (!bfd_malloc_and_get_section (input_bfd, input_section,
				     &elf_section_data (input_section)->
				     this_hdr.contents))
      return bfd_generic_get_relocated_section_contents (output_bfd,
							 link_info,
							 link_order, data,
							 relocatable,
							 symbols);


  symtab_hdr = &elf_tdata (input_bfd)->symtab_hdr;

  memcpy (data, elf_section_data (input_section)->this_hdr.contents,
	  (size_t) input_section->size);

  if ((input_section->flags & SEC_RELOC) != 0
      && input_section->reloc_count > 0)
    {
      asection **secpp;
      Elf_Internal_Sym *isym, *isymend;
      bfd_size_type amt;

      internal_relocs = (_bfd_elf_link_read_relocs
			 (input_bfd, input_section, (PTR) NULL,
			  (Elf_Internal_Rela *) NULL, FALSE));
      if (internal_relocs == NULL)
	goto error_return;

      if (symtab_hdr->sh_info != 0)
	{
	  isymbuf = (Elf_Internal_Sym *) symtab_hdr->contents;
	  if (isymbuf == NULL)
	    isymbuf = bfd_elf_get_elf_syms (input_bfd, symtab_hdr,
					    symtab_hdr->sh_info, 0,
					    NULL, NULL, NULL);
	  if (isymbuf == NULL)
	    goto error_return;
	}

      amt = symtab_hdr->sh_info;
      amt *= sizeof (asection *);
      sections = (asection **) bfd_malloc (amt);
      if (sections == NULL && amt != 0)
	goto error_return;

      isymend = isymbuf + symtab_hdr->sh_info;
      for (isym = isymbuf, secpp = sections; isym < isymend; ++isym, ++secpp)
	{
	  asection *isec;

	  if (isym->st_shndx == SHN_UNDEF)
	    isec = bfd_und_section_ptr;
	  else if (isym->st_shndx == SHN_ABS)
	    isec = bfd_abs_section_ptr;
	  else if (isym->st_shndx == SHN_COMMON)
	    isec = bfd_com_section_ptr;
	  else
	    isec = bfd_section_from_elf_index (input_bfd, isym->st_shndx);

	  *secpp = isec;
	}

      if (!elf32_m68hc05_relocate_section (output_bfd, link_info, input_bfd,
					   input_section, data,
					   internal_relocs, isymbuf,
					   sections))
	goto error_return;

      if (sections != NULL)
	free (sections);
      if (isymbuf != NULL
	  && symtab_hdr->contents != (unsigned char *) isymbuf)
	free (isymbuf);
      if (elf_section_data (input_section)->relocs != internal_relocs)
	free (internal_relocs);
    }

  return data;

error_return:
  if (sections != NULL)
    free (sections);
  if (isymbuf != NULL && symtab_hdr->contents != (unsigned char *) isymbuf)
    free (isymbuf);
  if (internal_relocs != NULL
      && elf_section_data (input_section)->relocs != internal_relocs)
    free (internal_relocs);
  return NULL;
}

typedef struct
{
  Elf_Internal_Shdr *symtab_hdr;
  Elf_Internal_Sym *isymbuf;
  asection *stab;
  Elf_Internal_Rela *stab_relocs;
} m68hc05_relax_state;

static int
compare_reloc (const void *e1, const void *e2)
{
  const Elf_Internal_Rela *i1 = (const Elf_Internal_Rela *) e1;
  const Elf_Internal_Rela *i2 = (const Elf_Internal_Rela *) e2;

  if (i1->r_offset == i2->r_offset)
    return 0;
  else
    return i1->r_offset < i2->r_offset ? -1 : 1;
}

static int
m68hc05_get_symvalue (bfd * abfd, Elf_Internal_Rela * irel, bfd_vma * symval,
		      m68hc05_relax_state * state)
{
  Elf_Internal_Sym *isym;
  asection *sym_sec;

  /* Get the value of the symbol referred to by the reloc.  */
  if (ELF32_R_SYM (irel->r_info) < state->symtab_hdr->sh_info)
    {
      /* A local symbol.  */
      isym = state->isymbuf + ELF32_R_SYM (irel->r_info);
      sym_sec = bfd_section_from_elf_index (abfd, isym->st_shndx);
      if (sym_sec == 0)
	*symval = isym->st_value;
      else
	*symval = (isym->st_value
		   + sym_sec->output_section->vma + sym_sec->output_offset);
    }
  else
    {
      unsigned long indx;
      struct elf_link_hash_entry *h;

      /* An external symbol.  */
      indx = ELF32_R_SYM (irel->r_info) - state->symtab_hdr->sh_info;
      h = elf_sym_hashes (abfd)[indx];
      BFD_ASSERT (h != NULL);
      if (h->root.type != bfd_link_hash_defined
	  && h->root.type != bfd_link_hash_defweak)
	{
	  /* This appears to be a reference to an undefined
	     symbol.  Just ignore it--it will be caught by the
	     regular reloc processing.  */
	  return -1;
	}

      isym = 0;
      sym_sec = h->root.u.def.section;
      *symval = (h->root.u.def.value
		 + sym_sec->output_section->vma + sym_sec->output_offset);
    }
  return 0;
}


/* Adjusts stab section */

static void
m68hc05_elf_relax_adjust_stabs (bfd * abfd, asection * sec,
				bfd_vma addr, int count,
				m68hc05_relax_state * state)
{
  bfd_byte *contents;
  bfd_vma pos, size;
  bfd_vma val, symval;
  bfd_vma func_start = 0;
  Elf_Internal_Rela *irel, *irelend;

  if (!state->stab)
    return;

  contents = elf_section_data (state->stab)->this_hdr.contents;

  size = state->stab->size;
  addr += sec->output_section->vma + sec->output_offset;
  irelend = state->stab_relocs + state->stab->reloc_count;

  for (pos = 0; pos < size; pos += 12)
    switch (contents[pos + 4])
      {
      case N_FUN:
	val = bfd_get_32 (abfd, contents + pos + 8);
	irel = state->stab_relocs;
	while (irel && irel < irelend && irel->r_offset < (pos) + 8)
	  irel++;
	symval = 0;
	if (irel && irel->r_offset == pos + 8)
	  {
	    if (m68hc05_get_symvalue (abfd, irel, &symval, state))
	      symval = 0;
	    if (ELF32_R_TYPE (irel->r_info) == R_M68HC05_32)
	      val = symval + irel->r_addend;
	  }
	func_start = val;
	break;
      case N_SLINE:
	val = bfd_get_32 (abfd, contents + pos + 8);
	if (func_start <= addr && addr + count <= val + func_start)
	  {
	    if (addr < val + func_start)
	      val -= count;
	    else
	      val = addr - func_start;
	    bfd_put_32 (abfd, val, contents + pos + 8);
	  }
      }
}

/* Delete some bytes from a section while relaxing.  */

static void
m68hc05_elf_relax_delete_bytes (bfd * abfd, asection * sec,
				bfd_vma addr, int count,
				m68hc05_relax_state * state)
{
  Elf_Internal_Shdr *symtab_hdr;
  unsigned int sec_shndx;
  bfd_byte *contents;
  Elf_Internal_Rela *irel, *irelend;
  bfd_vma toaddr;
  Elf_Internal_Sym *isymbuf, *isym, *isymend;
  struct elf_link_hash_entry **sym_hashes;
  struct elf_link_hash_entry **end_hashes;
  unsigned int symcount;

  m68hc05_elf_relax_adjust_stabs (abfd, sec, addr, count, state);

  symtab_hdr = &elf_tdata (abfd)->symtab_hdr;
  isymbuf = (Elf_Internal_Sym *) symtab_hdr->contents;

  sec_shndx = _bfd_elf_section_from_bfd_section (abfd, sec);

  contents = elf_section_data (sec)->this_hdr.contents;

  toaddr = sec->size;

  irel = elf_section_data (sec)->relocs;
  irelend = irel + sec->reloc_count;

  /* Actually delete the bytes.  */
  memmove (contents + addr, contents + addr + count,
	   (size_t) (toaddr - addr - count));

  sec->size -= count;

  /* Adjust all the relocs.  */
  for (irel = elf_section_data (sec)->relocs; irel < irelend; irel++)
    {
      unsigned long old_offset;

      old_offset = irel->r_offset;

      /* See if this reloc was for the bytes we have deleted, in which
         case we no longer care about it.  Don't delete relocs which
         represent addresses, though.  */
      if (irel->r_offset >= addr && irel->r_offset < addr + count)
	irel->r_info = ELF32_R_INFO (ELF32_R_SYM (irel->r_info),
				     R_M68HC05_NONE);

      /* Get the new reloc address.  */
      if ((irel->r_offset > addr && irel->r_offset < toaddr))
	irel->r_offset -= count;
    }

  /* Adjust the local symbols defined in this section.  */
  isymend = isymbuf + symtab_hdr->sh_info;
  for (isym = isymbuf; isym < isymend; isym++)
    {
      if (isym->st_shndx == sec_shndx
	  && isym->st_value > addr && isym->st_value <= toaddr)
	isym->st_value -= count;
    }

  /* Now adjust the global symbols defined in this section.  */
  symcount = (symtab_hdr->sh_size / sizeof (Elf32_External_Sym)
	      - symtab_hdr->sh_info);
  sym_hashes = elf_sym_hashes (abfd);
  end_hashes = sym_hashes + symcount;
  for (; sym_hashes < end_hashes; sym_hashes++)
    {
      struct elf_link_hash_entry *sym_hash = *sym_hashes;
      if ((sym_hash->root.type == bfd_link_hash_defined
	   || sym_hash->root.type == bfd_link_hash_defweak)
	  && sym_hash->root.u.def.section == sec
	  && sym_hash->root.u.def.value > addr
	  && sym_hash->root.u.def.value <= toaddr)
	{
	  sym_hash->root.u.def.value -= count;
	}
    }
}


		  /* Note that we've changed the relocation contents, etc.  */
#define PUBLISH_RELAX                                                   \
                  elf_section_data (sec)->relocs = internal_relocs;     \
                  free_relocs = NULL;                                   \
		                                                        \
                  elf_section_data (sec)->this_hdr.contents = contents; \
                  free_contents = NULL;                                 \
		                                                        \
                  state.symtab_hdr->contents = (bfd_byte *) state.isymbuf;\
                  free_extsyms = NULL;



static bfd_boolean m68hc05_elf_relax_section
  (bfd * abfd, asection * sec, struct bfd_link_info *link_info,
   bfd_boolean * again)
{
  Elf_Internal_Shdr *shndx_hdr;
  Elf_Internal_Rela *internal_relocs;
  Elf_Internal_Rela *free_relocs = NULL;
  Elf_Internal_Rela *irel, *irelend;
  bfd_byte *contents = NULL;
  bfd_byte *free_contents = NULL;
  Elf32_External_Sym *free_extsyms = NULL;
  m68hc05_relax_state state;

  memset (&state, 0, sizeof (state));

  /* Assume nothing changes.  */
  *again = FALSE;

  /* We don't have to do anything for a relocatable link, if
     this section does not have relocs, or if this is not a
     code section.  */
  if (link_info->relocatable
      || (sec->flags & SEC_RELOC) == 0
      || sec->reloc_count == 0 || (sec->flags & SEC_CODE) == 0)
    return TRUE;

  state.symtab_hdr = &elf_tdata (abfd)->symtab_hdr;
  shndx_hdr = &elf_tdata (abfd)->symtab_shndx_hdr;
  state.stab = bfd_get_section_by_name (abfd, ".stab");

  if (state.stab)
    {
      bfd_byte *stab_contents = NULL;
      if (elf_section_data (state.stab)->this_hdr.contents == NULL)
	{
	  if (!bfd_malloc_and_get_section (abfd, state.stab, &stab_contents))
	    state.stab = 0;
	  else
	    elf_section_data (state.stab)->this_hdr.contents = stab_contents;
	}
    }

  if (state.stab && state.stab->reloc_count > 0)
    {
      state.stab_relocs = (_bfd_elf_link_read_relocs
			   (abfd, state.stab, (PTR) NULL,
			    (Elf_Internal_Rela *) NULL,
			    link_info->keep_memory));
      if (state.stab_relocs)
	qsort (state.stab_relocs, state.stab->reloc_count,
	       sizeof (Elf_Internal_Rela), compare_reloc);
    }

  /* Get a copy of the native relocations.  */
  internal_relocs = (_bfd_elf_link_read_relocs
		     (abfd, sec, (PTR) NULL, (Elf_Internal_Rela *) NULL,
		      link_info->keep_memory));
  if (internal_relocs == NULL)
    goto error_return;
  if (!link_info->keep_memory)
    free_relocs = internal_relocs;

  /* Checking for branch relaxation relies on the relocations to
     be sorted on 'r_offset'.  This is not guaranteed so we must sort.  */
  qsort (internal_relocs, sec->reloc_count, sizeof (Elf_Internal_Rela),
	 compare_reloc);

  /* Walk through them looking for relaxing opportunities.  */
  irelend = internal_relocs + sec->reloc_count;
  for (irel = internal_relocs; irel < irelend; irel++)
    {
      bfd_vma symval;
      bfd_vma value;
      unsigned char code;
      bfd_vma addr;
      bfd_vma code_addr;

      /* If this isn't something that can be relaxed, then ignore
         this reloc.  */
      if (ELF32_R_TYPE (irel->r_info) != (int) R_M68HC05_RELAX_16
	  && ELF32_R_TYPE (irel->r_info) != (int) R_M68HC05_RELAX_GROUP
	  && ELF32_R_TYPE (irel->r_info) != (int) R_M68HC05_RELAX_PCREL_8
	  && ELF32_R_TYPE (irel->r_info) != (int) R_M68HC05_RELAX_8)
	continue;

      /* Get the section contents if we haven't done so already.  */
      if (contents == NULL)
	{
	  /* Get cached copy if it exists.  */
	  if (elf_section_data (sec)->this_hdr.contents != NULL)
	    contents = elf_section_data (sec)->this_hdr.contents;
	  else
	    {
	      /* Go get them off disk.  */
	      if (!bfd_malloc_and_get_section (abfd, sec, &contents))
		goto error_return;
	    }
	}

      /* Read this BFD's symbols if we haven't done so already.  */
      if (state.isymbuf == NULL && state.symtab_hdr->sh_info != 0)
	{
	  state.isymbuf = (Elf_Internal_Sym *) state.symtab_hdr->contents;
	  if (state.isymbuf == NULL)
	    state.isymbuf = bfd_elf_get_elf_syms (abfd, state.symtab_hdr,
						  state.symtab_hdr->sh_info,
						  0, NULL, NULL, NULL);
	  if (state.isymbuf == NULL)
	    goto error_return;
	}

      /* Get the value of the symbol referred to by the reloc.  */
      if (m68hc05_get_symvalue (abfd, irel, &symval, &state))
	continue;

      value = symval + irel->r_addend;
      addr = irel->r_offset;
      code = bfd_get_8 (abfd, contents + addr - 1);
      code_addr = addr + sec->output_section->vma + sec->output_offset;

      if (ELF32_R_TYPE (irel->r_info) == (int) R_M68HC05_RELAX_GROUP)
	{
	  Elf_Internal_Rela *irel_jmp;

	  /*brset/brclr group */
	  code = bfd_get_8 (abfd, contents + addr);
	  if ((code & 0xf0) == 0x00 && abs (value - code_addr) <= 125)
	    {
	      irel_jmp = irel + 1;
	      if (irel_jmp->r_offset < addr + 2)
		irel_jmp++;
	      if (irel_jmp->r_offset != addr + 2)
		abort ();
	      irel_jmp++;
	      if (irel_jmp->r_offset != addr + 4)
		abort ();
	      code = code ^ 1;
	      bfd_put_8 (abfd, code, contents + addr);
	      PUBLISH_RELAX;
	      irel->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info), R_M68HC05_NONE);
	      m68hc05_elf_relax_delete_bytes (abfd, sec, addr + 2, 2, &state);
	      if (ELF32_R_TYPE (irel_jmp->r_info) == (int) R_M68HC05_RELAX_16)
		m68hc05_elf_relax_delete_bytes (abfd, sec, addr + 3, 1,
						&state);
	      irel_jmp->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info), R_M68HC05_PCREL_8);
	      *again = TRUE;
	      continue;
	    }
	  if ((code & 0xf0) == 0x20 && abs (value - code_addr) <= 125)
	    {
	      irel_jmp = irel + 1;
	      if (irel_jmp->r_offset != addr + 1)
		abort ();
	      irel_jmp++;
	      if (irel_jmp->r_offset != addr + 3)
		abort ();
	      code = code ^ 1;
	      bfd_put_8 (abfd, code, contents + addr);
	      PUBLISH_RELAX;
	      irel->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info), R_M68HC05_NONE);
	      m68hc05_elf_relax_delete_bytes (abfd, sec, addr + 1, 2, &state);
	      if (ELF32_R_TYPE (irel_jmp->r_info) == (int) R_M68HC05_RELAX_16)
		m68hc05_elf_relax_delete_bytes (abfd, sec, addr + 2, 1,
						&state);
	      irel_jmp->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info),
			      R_M68HC05_RELAX_PCREL_8);
	      *again = TRUE;
	      continue;
	    }
	}
      if (ELF32_R_TYPE (irel->r_info) == (int) R_M68HC05_RELAX_16)
	{
	  /*JSR EXT->bsr */
	  if (code == 0xCD && abs (value - code_addr) <= 125)
	    {
	      code = 0xAD;
	      bfd_put_8 (abfd, code, contents + addr - 1);
	      irel->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info),
			      R_M68HC05_RELAX_PCREL_8);
	      PUBLISH_RELAX;
	      m68hc05_elf_relax_delete_bytes (abfd, sec, addr + 1, 1, &state);
	      *again = TRUE;
	      continue;
	    }
	  /*JMP EXT -> bra */
	  if (code == 0xCC && abs (value - code_addr) <= 125)
	    {
	      code = 0x20;
	      bfd_put_8 (abfd, code, contents + addr - 1);
	      irel->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info),
			      R_M68HC05_RELAX_PCREL_8);
	      PUBLISH_RELAX;
	      m68hc05_elf_relax_delete_bytes (abfd, sec, addr + 1, 1, &state);
	      *again = TRUE;
	      continue;
	    }
	  /*IX2->IX0 */
	  if ((code & 0xf0) == 0xd0 && value == 0)
	    {
	      code = (code & 0x0f) | 0xF0;
	      bfd_put_8 (abfd, code, contents + addr - 1);
	      irel->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info), R_M68HC05_NONE);
	      PUBLISH_RELAX;
	      m68hc05_elf_relax_delete_bytes (abfd, sec, addr, 2, &state);
	      *again = TRUE;
	      continue;
	    }
	  /*IX2->IX1 */
	  if ((code & 0xf0) == 0xd0 && value <= 0xff)
	    {
	      code = (code & 0x0f) | 0xe0;
	      bfd_put_8 (abfd, code, contents + addr - 1);
	      irel->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info), R_M68HC05_RELAX_8);
	      PUBLISH_RELAX;
	      m68hc05_elf_relax_delete_bytes (abfd, sec, addr + 1, 1, &state);
	      *again = TRUE;
	      continue;
	    }
	  /*EXT->DIR */
	  if ((code & 0xf0) == 0xc0 && value <= 0xff)
	    {
	      code = (code & 0x0f) | 0xb0;
	      bfd_put_8 (abfd, code, contents + addr - 1);
	      irel->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info), R_M68HC05_RELAX_8);
	      PUBLISH_RELAX;
	      m68hc05_elf_relax_delete_bytes (abfd, sec, addr + 1, 1, &state);
	      *again = TRUE;
	      continue;
	    }
	}
      if (ELF32_R_TYPE (irel->r_info) == (int) R_M68HC05_RELAX_8)
	{

	  /*IX1->IX0 */
	  if ((code & 0xf0) == 0x60 && value == 0)
	    {
	      code = (code & 0x0f) | 0x70;
	      bfd_put_8 (abfd, code, contents + addr - 1);
	      irel->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info), R_M68HC05_NONE);
	      PUBLISH_RELAX;
	      m68hc05_elf_relax_delete_bytes (abfd, sec, addr, 1, &state);
	      *again = TRUE;
	      continue;
	    }
	}
      if (ELF32_R_TYPE (irel->r_info) == (int) R_M68HC05_RELAX_PCREL_8)
	{
	  /* delete brn */
	  if (code == 0x21)
	    {
	      irel->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info), R_M68HC05_NONE);
	      PUBLISH_RELAX;
	      m68hc05_elf_relax_delete_bytes (abfd, sec, addr - 1, 2, &state);
	      *again = TRUE;
	      continue;
	    }
	  /* delete bra next instruction */
	  if (code == 0x20 && value - code_addr == 1)
	    {
	      irel->r_info =
		ELF32_R_INFO (ELF32_R_SYM (irel->r_info), R_M68HC05_NONE);
	      PUBLISH_RELAX;
	      m68hc05_elf_relax_delete_bytes (abfd, sec, addr - 1, 2, &state);
	      *again = TRUE;
	      continue;
	    }
	}
    }

  if (free_relocs != NULL)
    {
      free (free_relocs);
      free_relocs = NULL;
    }

  if (free_contents != NULL)
    {
      if (!link_info->keep_memory)
	free (free_contents);
      else
	{
	  /* Cache the section contents for elf_link_input_bfd.  */
	  elf_section_data (sec)->this_hdr.contents = contents;
	}
      free_contents = NULL;
    }

  if (free_extsyms != NULL)
    {
      if (!link_info->keep_memory)
	free (free_extsyms);
      else
	{
	  /* Cache the symbols for elf_link_input_bfd.  */
	  state.symtab_hdr->contents = (unsigned char *) state.isymbuf;
	}
      free_extsyms = NULL;
    }
  if (state.stab_relocs && !link_info->keep_memory)
    free (state.stab_relocs);

  return TRUE;

error_return:
  if (free_relocs != NULL)
    free (free_relocs);
  if (free_contents != NULL)
    free (free_contents);
  if (free_extsyms != NULL)
    free (free_extsyms);
  if (state.stab_relocs && !link_info->keep_memory)
    free (state.stab_relocs);
  return FALSE;
}


#define ELF_ARCH		bfd_arch_m68hc05
#define ELF_MACHINE_CODE	EM_68HC05
#define ELF_MAXPAGESIZE		1

#define TARGET_BIG_SYM       bfd_elf32_m68hc05_vec
#define TARGET_BIG_NAME	"elf32-m68hc05"

#define bfd_elf32_bfd_relax_section          m68hc05_elf_relax_section
#define elf_info_to_howto	             m68hc05_info_to_howto_rel
#define elf_info_to_howto_rel	             NULL
#define elf_backend_relocate_section         elf32_m68hc05_relocate_section
#define elf_backend_gc_mark_hook             elf32_m68hc05_gc_mark_hook
#define elf_backend_gc_sweep_hook            elf32_m68hc05_gc_sweep_hook
#define elf_backend_check_relocs             elf32_m68hc05_check_relocs
#define bfd_elf32_bfd_get_relocated_section_contents elf32_m68hc05_get_relocated_section_contents
#define elf_backend_can_gc_sections          1
#define elf_backend_rela_normal		     1
#define elf_backend_final_write_processing 0
#define elf_backend_object_p		0

#include "elf32-target.h"
